/* Demon.c: cyclic system described by Dewdney in Sci Am August 89 */
/*   Microsoft QuickC or 5.x C Compiler.  August, 1989; B Katzung  */
#include "stdio.h"
#include "graph.h"
#include "conio.h"
#include "bios.h"

#define  f1 15104  /* function key readouts for help, matrix, etc */
#define  f2 15360
#define  f3 15616
#define  f4 15872
#define  f5 16128
#define  f6 16384
#define  f7 16640
#define  f8 16896
#define  f9 17152
#define  f10   17408
#define  esc   283
int huge  old[200][200];  /* 2 arrays for the automaton matrix */
int huge  new[200][200];
int pos, state, max;
int ka,kb,la,lb;
int r,num,ct,mct,vga,key,oldgcolor,oldcolor,flip,freeze;
long j;
char statebuf[4],buf[80];

main(argc,argv)
int argc;
char *argv[];
{
	register int x,y;
	struct videoconfig config;
	/* First make sure the display is a VGA or EGA graphics display */
	if (_setvideomode(_VRES16COLOR) != 0) 
		vga = 80;
	else
		if(_setvideomode(_ERESCOLOR) != 0)
			vga = 0;
	/* otherwise exit gracefully */
		else  {  
			printf("Program requires EGA or VGA 16 color display\n");
			exit(0);
		}
	_getvideoconfig(&config);
	if(argc < 3)   {
		printf("Demon is a cyclic automaton, from AK Dewdney, Sci American, Aug 1989\n");
		printf("Starting from a random pattern, it generates a highly organized pattern.\n");
		printf("Number of states (colors), number of cycles, and size can also be entered\n");
		printf("on the command line; eg, demon 8 100 100\n");
		printf("Enter number of states (colors) (8 - 20, 8 is best) > ");
		scanf("%d", &state);
		printf("Enter num of iterations (50 - 600) > ");
		scanf("%d",&num);
		printf("Enter array size for figure (50 - 200),\n"
		    "     small is faster, but use 120 for states > 14) > ");
		scanf("%d",&max);
      _clearscreen(_GCLEARSCREEN);
	}
	else  {
		state = atoi(argv[1]);
		num = atoi(argv[2]);
		max = atoi(argv[3]);
	}
   if(max > 200) max = 200;
	_getvideoconfig(&config);
	printf("The Demon Automaton (Sci American, Aug 89): states=%d, cycles=%d, size=%d\n", state,num,max);
	for (y = 0; y < max; y++)  { /* set random state in each cell */
		for (x = 0; x < max; x++)  {
			new[x][y] = old[x][y] = rand(1) % state;
			_setcolor((old[x][y]) * 3 % 16);
			_setpixel(x+350,y + 40 + vga);
		} /* end for x .. */
	} /* end for y .. */
	freeze = 0;
	/* start cyclic activity */
	while (ct++ < num-1)  {
		freeze = 0;  /* clean up as soon as unfrozen */
		_settextposition(2,2);
		sprintf(buf,"                                        ");
		_outtext(buf);
		_settextposition(20,45);  /* keep track of number of cycles finished */
		sprintf(buf,"%d cycles finished", ct+1);
		_outtext(buf);
		_settextposition(25,1);  /* display function key meanings at bottom */
		sprintf(buf,"F1,F2:help on,off  F3,F4:matrix on,stop  F5,F6:freeze,restart  F10 or Esc:Quit");
		_outtext(buf);

		for (y = 0; y < max; y++)  { /* start cycling thru the rows (y axis) */
			if(y == 0)  lb = max-1; /* if first or last row, wrap-around */
			else lb = y-1;        /* lb is neighbor below the cell being checked */
			if(y == max-1) la = 0; /* la is the one above the cell being checked */
			else la = y+1;
			for (x = 0; x < max; x++)  {
				/* check the function keys for input */
				if( (key = (_bios_keybrd(_KEYBRD_READY))) != 0)   {
					if(key == esc) goto clear; /* time to quit */
					if(key == f1) gethelp();
					if(key == f2) helpoff();
					if(key == f3) {
                  flip = 1; mct = ct; 
                  } /* matrix on */
               if(mct == ct)   matdisp();
					if(key == f4) flip = 0; /* matrix off */
					if(key == f5)  { /* stop the cycling */
						freeze = 1 ;
						_settextposition(2,2);
						oldcolor = _gettextcolor();
						_settextcolor(12);
						sprintf(buf,"Cycling Frozen; press F6 to restart");
						_outtext(buf);
						_settextcolor(oldcolor);
                   while(key = (_bios_keybrd(_KEYBRD_READ)) != f6);
					}
					if(key == f6) freeze = 0; /* restart the automaton */
					if(key == f10) goto clear; /* time to quit */
					getch(); /* clean up the keyboard buffer */
					key = 0;
				}
				/* finished checking function keys */
				if(x == 0) {   /* check columns (x axis); if first or last, wrap */
					ka = x+1; /* ka is neighbor to left of cell */
					kb = max-1; /* kb is to right of cell */
				}
				else if(x > 0 && x < max-1)  {
					ka = x+1;
					kb = x-1;
				}
				else if(x == max-1) {
					ka = 0;
					kb = x-1;
				}
				/* start checking neighbors against the cell at coordinates x,y */
				if(old[x][lb] == (old[x][y] + 1)% state ) { /* cell below */
					new[x][y] = old[x][lb] ;
				   _setcolor((new[x][y]) * 3 % 16);
				   _setpixel(x+350,y + 40 + vga);
               if(flip == 1 && x < 10 && y < 10) {
                  _settextposition(y+6,x*3+9);
                  sprintf(statebuf,"%3d",new[x][y]);
                  _outtext(statebuf);
               }
				}
				if(old[kb][y] == (old[x][y] + 1)% state ) { /* cell to right */
					new[x][y] = old[kb][y] ;
   				_setcolor((new[x][y]) * 3 % 16);
	   			_setpixel(x+350,y + 40 + vga);
               if(flip == 1 && x < 10 && y < 10) {
                  _settextposition(y+6,x*3+9);
                  sprintf(statebuf,"%3d",new[x][y]);
                  _outtext(statebuf);
               }
				}
				if(old[ka][y] == (old[x][y] + 1)% state ) { /* cell to left */
					new[x][y] = old[ka][y] ;
   				_setcolor((new[x][y]) * 3 % 16);
	   			_setpixel(x+350,y + 40 + vga);
               if(flip == 1 && x < 10 && y < 10) {
                  _settextposition(y+6,x*3+9);
                  sprintf(statebuf,"%3d",new[x][y]);
                  _outtext(statebuf);
               }
				}
				if(old[x][la] == (old[x][y] + 1)% state ) { /* cell above */
					new[x][y] = old[x][la] ;
   				_setcolor((new[x][y]) * 3 % 16);
	   			_setpixel(x+350,y + 40 + vga);
               if(flip == 1 && x < 10 && y < 10) {
                  _settextposition(y+6,x*3+9);
                  sprintf(statebuf,"%3d",new[x][y]);
                  _outtext(statebuf);
               }
				}
				/* end of checking the 4 adjoining neighbor cells */
			} /* end for x */
		} /* end for y */
		/* update the state of each cell */
		for (y = 0; y < max; y++)   {
			for (x = 0; x < max; x++)  {
            if(flip == 1 && mct == ct && x < 10 && y < 10) {
               _settextposition(y+6,x*3+9);
               sprintf(statebuf,"%3d",new[x][y]);
               _outtext(statebuf);
            }
				old[x][y] = new[x][y];
			} /* end for x */
		} /* end for y */
 	}
	/* end while ct, finished requested number of cycles */
	_settextposition(2,5);
	_settextcolor(9);
	sprintf(buf,"Finished; Colors (not cell states) will cycle; press any key to exit");
	_outtext(buf);
	_settextcolor(oldcolor);
	getch();   
	while( !kbhit() ) {  /* cycle colors after requested iterations have finished */
		for(y = 0; y < max; y++)   {
			for (x = 0; x < max; x++)  {
				_setcolor((new[x][y]) * 3 % 16);
				_setpixel(x+350,y + 40 + vga);
				new[x][y] = (new[x][y]+1) % 16;
			} /* end for x */
		} /* end for y */
		for(j = 0;j < 1000000;j++);
	}

clear: /* time to quit */
	getch(); /* clear keyboard buffer */
	_clearscreen(_GCLEARSCREEN);
   _displaycursor(_GCURSORON);
	_setvideomode(_DEFAULTMODE);     
}	/* end of main */

gethelp() /* simple-minded help screen */
{
	_settextposition(18,2);
	sprintf(buf,"A cell increments its state (color) when");
	_outtext(buf);
	_settextposition(19,2);
	sprintf(buf,"the state of any side-adjoining cell (not");
	_outtext(buf);
	_settextposition(20,2);
	sprintf(buf,"a corner-adjoining cell) is one greater");
	_outtext(buf);
	_settextposition(21,2);
	sprintf(buf,"than its own state.  Matrix shows the ");
	_outtext(buf);
	_settextposition(22,2);
	sprintf(buf,"states (colors) of the upper left corner.");
	_outtext(buf);
}

helpoff() /* simple-minded way to clear help text */
{
	_settextposition(18,2);
	sprintf(buf,"                                         ");
	_outtext(buf);
	_settextposition(19,2);
	sprintf(buf,"                                         ");
	_outtext(buf);
	_settextposition(20,2);
	sprintf(buf,"                                         ");
	_outtext(buf);
	_settextposition(21,2);
	sprintf(buf,"                                         ");
	_outtext(buf);
	_settextposition(22,2);
	sprintf(buf,"                                         ");
	_outtext(buf);
}

matdisp()
{
		if (flip == 1) {
			/* display the matrix of cell states for cells 1,1 to 5,5 (ignores 0,0) */
			/* matrix of values for cells 1,1 - 5,5 */
         oldgcolor = _getcolor();
         _setcolor(0);
         _setlinestyle(0xFFFF);
         _rectangle(_GBORDER,0+349,0+39+vga,11+350,11+40+vga);
         _setcolor(15);
         _rectangle(_GBORDER,0+348,0+38+vga,11+351,11+41+vga);
         _setcolor(oldgcolor);
			_settextposition(4,2);
			sprintf(buf,"Columns: 0  1  2  3  4  5  6  7  8  9");
			_outtext(buf);
			_settextposition(5,2);
			sprintf(buf,"                   State");
			_outtext(buf);
         for(r = 0;r < 10;r++)   {
   			_settextposition(r+6,2);
	   		sprintf(buf,"Row%1d",r);
		   	_outtext(buf);
            }
		} /* end of if flip == 1 */

}
/* end of program demon.c */
